"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const resolve_from_1 = __importDefault(require("resolve-from"));
const utils_1 = require("./utils");
const fileToIdMap = new Map();
function resolveModule(id, cwd, res) {
    let modulePath;
    let sourceMapPath = undefined;
    // TODO support custom imports map e.g. for snowpack web_modules
    if (id.endsWith('.map')) {
        sourceMapPath = id;
        id = fileToIdMap.get(id.replace(/\.map$/, ''));
        if (!id) {
            res.statusCode = 404;
            res.end();
            return;
        }
    }
    // fallback to node resolve
    try {
        modulePath = resolve_from_1.default(cwd, `${id}/package.json`);
        if (id === 'vue') {
            modulePath = path_1.default.join(path_1.default.dirname(modulePath), 'dist/vue.runtime.esm-browser.js');
        }
        else {
            // module resolved, try to locate its "module" entry
            const pkg = require(modulePath);
            modulePath = path_1.default.join(path_1.default.dirname(modulePath), pkg.module || pkg.main);
            fileToIdMap.set(path_1.default.basename(modulePath), id);
            // this is a source map request.
            if (sourceMapPath) {
                modulePath = path_1.default.join(path_1.default.dirname(modulePath), sourceMapPath);
            }
        }
        utils_1.sendJSStream(res, modulePath);
    }
    catch (e) {
        console.error(e);
        res.statusCode = 404;
        res.end();
    }
}
exports.resolveModule = resolveModule;
